PROGRAM ImageJ;

//
//  Author: George H. Silva <George.Silva@chemie.bio.uni-giessen.de>
//  Version History:
//    v1.0.0.x - 2006/1/21 - initial release
//             - Basic code based on "The Java launcher for Windows v1.22" by Jacob Marner
//             - Improved/customized for ImageJ (i.e. can handle file-associations, etc)
//             - This windows stub can be used for other Java applications as well;
//               just change the icon, set the DEF_CFG constant and re-compile!
//
//    v1.1.0.x - 2006/1/23 - many updates/improvements
//             - Robust loader to handle various config scenarios
//             - Code to auto-create the config file
//             - Code for auto-detection of various parameters (JVM, mem usage, etc)
//             - Code to allow selection of needed files
//             - This version essentially can reconfigure itself as needed, depending
//               on how the user changes their JAVA install
//
//    v1.2.0.x - 2006/1/24 - minor updates/improvements
//             - Bug fixes and logic changes per Wayne's suggestions
//
//    v1.3.0.x - 2006/1/24 - minor updates/improvements
//             - Correctly expands command-line params to full-paths as needed
//             - Preserves relative path for JRE if possible
//             - No longer needlessly expands relative paths and re-saves CFG file
//             - Improved logic to make sure tools.jar is in JVM params
//

USES
  Windows, SysUtils, ShellAPI, Shared;

{$R *.res}

VAR
  nSaveCfg: Integer;
  sCfg,sIJDir,sJVM,sParams,sJVMEx,sParamsEx: String;

BEGIN
  // Read/Write mode for files (global)
  FileMode:=2;
  nSaveCfg:=0;
  // Look for the config file - default to DEF_CFG or create one as needed
  if not(FileExists(Evalset(ChangeFileExt(ParamStr(0),CFG_EXT),sCfg))) then
    begin
      if SameText(ExtractFileName(sCfg),DEF_CFG) then
        sCfg:=CreateCfgFile('','','','',true) // Create a new CFG file
      else // Try default CFG
        if not(FileExists(EvalSet(ExtractFilePath(ParamStr(0))+DEF_CFG,sCfg))) then
          sCfg:=CreateCfgFile('','','','',true) // Create a new CFG file
    end;
  // Read config and auto-repair if needed
  if not(ReadCfgFile(sCfg,sIJDir,sJVM,sParams)) then
    begin
      if (MessageBox(0,PChar('The configuration file is corrupt. Would you like to '+
                     'auto-repair this file?'),PChar('Bad Configuration File Format'),
                     MB_YESNO or MB_ICONQUESTION) = ID_NO) then
        FatalError('The configuration file should contain at least 3 lines:'+#13#13+
                   'Line 1-  The '+TXT_APP+' install directory (location of JAR file)'+#13+
                   'Line 2-  The location of the java virtual machine (JVM) executable'+#13+
                   'Line 3-  Parameters to pass to the JVM'+#13#13+
                   'To restore the default configuration, simply re-start '+TXT_APP+' after '+
                   'deleting this file:'+#13#13+sCfg,
                   'Bad Configuration File Format',2)
      else
        begin
          sCfg:=CreateCfgFile(sCfg,'','','');
          if not(ReadCfgFile(sCfg,sIJDir,sJVM,sParams)) then
            FatalError('The configuration file could not be repaired. Please check the '+
                       TXT_APP+' installation and try again','Bad Configuration File Format',3)
        end;
    end;
  // Validate the ImageJ folder - sIJDir should be expanded at this point
  if not(IJJarExists(sIJDir)) then
    begin
      if GetValidIJPath(sIJDir,false) then
        Inc(nSaveCfg) // Save new data below if all is ok
      else
        FatalError('Could not locate the '+TXT_APP+' JAR file ('+JAR_IMAGEJ+'). Please '+
                   'check the installation and try again.','Fatal Error',4);
    end;
  // Validate the JVM; at this point:
  //   sJVM  = relative OR full path to JVM
  //  sJVMEx = actual full path to JVM
  sJVMEx:=sJVM;
  if not(FileExists(sJVMEx)) then
    begin
      // Save new data below if all is ok
      if GetValidJVM(sIJDir,sJVMEx) then // This will expand sJVMEx as needed
        begin
          // Verify it was not a relative path
          if not(SameText(sJVM,RelativeJVM(sIJDir,sJVMEx))) then
            begin
              sJVM:=sJVMEx; // New path
              Inc(nSaveCfg);
            end;
        end
      else
        FatalError('Could not locate the JVM file ('+EXE_JVM+'). Please '+
                   'check the installation and try again.','Fatal Error',5);
    end;
  // Get expanded (passed) parameters; NOTE: do this HERE before switching directories
  sParamsEx:=Trim(sParams+' '+GetCmdLnParamsEx);
  // Change to selected directory
  if not(SetCurrentDir(sIJDir)) then
    FatalError('Problem setting the current directory to: "'+sIJDir+'"','Directory Error',6);
  // Run the program
  if (ShellExecute(0,nil,PChar(sJVMEx),PChar(sParamsEx),PChar(sIJDir),SW_SHOWNORMAL) <=32) then
    FatalError('Could not launch '+TXT_APP+' as configured. Please check the '+
               'settings and try again.','Fatal Error',7)
  else
    if (nSaveCfg > 0) then // Save this if needed
      CreateCfgFile(sCfg,sIJDir,sJVM,sParams);
END.

